﻿using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.LiveIntegration.EndpointMonitoring;
using Dynamicweb.Ecommerce.LiveIntegration.Licensing;
using Dynamicweb.Ecommerce.LiveIntegration.Logging;
using Dynamicweb.Extensibility.Notifications;
using System;
using System.Xml;
using static Dynamicweb.Ecommerce.LiveIntegration.Notifications.Communication;

namespace Dynamicweb.Ecommerce.LiveIntegration.Connectors
{
    /// <summary>
    /// Main class to interact with a remote ERP.
    /// </summary>
    internal static class Connector
    {
        /// <summary>
        /// The maximum retry count
        /// </summary>
        private static readonly int MaxRetryCount = 3;

        /// <summary>
        /// The retry interval
        /// </summary>
        private static readonly int RetryInterval = 500;

        public static bool EnableThrowExceptions
        {
            get
            {
                return Core.Converter.ToBoolean(Context.Current?.Items?["LiveIntegrationAddInThrowExceptions"]);
            }
            set
            {
                if (Context.Current?.Items != null)
                {
                    Context.Current.Items["LiveIntegrationAddInThrowExceptions"] = value;
                }
            }
        }

        private static ConnectorBase _currentConnector = null;
        private static ConnectorBase CurrentConnector
        {
            get
            {
                if (_currentConnector == null)
                {
                    if (!string.IsNullOrEmpty(Settings.Instance.Endpoint))
                    {
                        _currentConnector = new EndpointConnector();
                    }
                    else
                    {
                        _currentConnector = new WebServiceConnector();
                    }
                }
                return _currentConnector;
            }
        }
        //private static EndpointConnector EndpointConnector = 
        //private static WebServiceConnector WebServiceConnector = new WebServiceConnector();

        /// <summary>
        /// Calculates the order.
        /// </summary>
        /// <param name="orderXml">The order XML.</param>
        /// <param name="orderId">The order identifier.</param>
        /// <param name="createOrder">if set to <c>true</c> [create order].</param>
        /// <param name="error">The error.</param>
        /// <returns>XmlDocument.</returns>
        public static XmlDocument CalculateOrder(string orderXml, string orderId, bool createOrder, out Exception error)
        {
            error = null;
            XmlDocument document = null;

            try
            {
                Diagnostics.ExecutionTable.Current.Add("LiveIntegration.Connector.CalculateOrder START");
                // only retry if is not create order, for create order schedule task will send later
                document = Communicate(orderXml, $"CalculateOrder (ID: {orderId}, CreateOrder: {createOrder})", !createOrder, true);
                Diagnostics.ExecutionTable.Current.Add("LiveIntegration.Connector.CalculateOrder END");
            }
            catch (Exception ex)
            {
                error = ex;
            }

            if (EnableThrowExceptions && error != null)
            {
                throw error;
            }

            return document;
        }

        /// <summary>
        /// Clears the cache.
        /// </summary>
        public static void ClearCache()
        {
            CurrentConnector.ClearCache();
        }

        /// <summary>
        /// Gets the products information.
        /// </summary>
        /// <param name="productsXml">The products XML.</param>
        /// <param name="retry">Retry request on error.</param>
        /// <returns>XmlDocument.</returns>
        public static XmlDocument GetProductsInfo(string productsXml, bool retry = false)
        {
            Diagnostics.ExecutionTable.Current.Add("LiveIntegration.Connector.GetProductsInfo START");
            var result = Communicate(productsXml, "GetProductsInfo", retry); ;
            Diagnostics.ExecutionTable.Current.Add("LiveIntegration.Connector.GetProductsInfo END");
            return result;
        }

        /// <summary>
        /// Determines whether the web service connection is available.
        /// </summary>
        public static bool IsWebServiceConnectionAvailable()
        {
            bool result = CurrentConnector.IsWebServiceConnectionAvailable(null);
            return result;
        }

        /// <summary>
        /// Determines whether the web service connection is available.
        /// </summary>
        public static bool IsWebServiceConnectionAvailable(EndpointInfo endpoint)
        {
            bool result = CurrentConnector.IsWebServiceConnectionAvailable(endpoint);
            return result;
        }

        /// <summary>
        /// Retrieves data from the request string.
        /// </summary>
        /// <param name="request">The request.</param>
        /// <returns>XmlDocument.</returns>
        public static XmlDocument RetrieveDataFromRequestString(string request)
        {
            Diagnostics.ExecutionTable.Current.Add("LiveIntegration.Connector.RetrieveDataFromRequestString START");
            var result = Communicate(request, "RetrieveDataFromRequestString");
            Diagnostics.ExecutionTable.Current.Add("LiveIntegration.Connector.RetrieveDataFromRequestString END");
            return result;
        }

        /// <summary>
        /// Communicates the specified request.
        /// </summary>
        /// <param name="request">The request.</param>
        /// <param name="referenceName">Name of the reference.</param>
        /// <param name="retry">if set to <c>true</c> [retry].</param>
        /// <param name="throwException">if set to <c>true</c> [throw exception].</param>
        /// <returns>XmlDocument.</returns>
        private static XmlDocument Communicate(string request, string referenceName, bool retry = true, bool throwException = false)
        {
            XmlDocument result = null;
            int retryCount = 0;

            // if ERP is down and in retry mode wait!
            while (!IsWebServiceConnectionAvailable())
            {
                if (!retry || retryCount >= MaxRetryCount)
                {
                    return null;
                }

                ++retryCount;
                System.Threading.Thread.Sleep(RetryInterval);
            }

            var endpoint = CurrentConnector.GetEndpoint();
            string url = endpoint?.GetUrl();

            if (!LicenseService.IsLicenseValid(url))
            {
                Logger.Instance.Log(ErrorLevel.DebugInfo, $"Request send failed. License is not valid.");
                return null;
            }

            Exception exception = null;
            do
            {
                exception = null;
                string erpXmlResponse = null;
                try
                {
                    NotificationManager.Notify(OnBeforeErpCommunication, new OnBeforeErpCommunicationArgs(request, referenceName));

                    Logger.Instance.Log(ErrorLevel.DebugInfo, $"Request {referenceName} sent: '{request}'.");

                    erpXmlResponse = CurrentConnector.Execute(endpoint, request);

                    if (!string.IsNullOrEmpty(erpXmlResponse))
                    {
                        Logger.Instance.Log(ErrorLevel.DebugInfo, $"Response {referenceName} received: '{erpXmlResponse}'.");
                        if (!Helpers.ParseResponseToXml(erpXmlResponse, out result))
                        {
                            result = null;
                            exception = new Exception("Response is not valid XML");
                        }
                        if (result != null && !LicenseService.ValidateLicense(url, result))
                        {
                            Logger.Instance.Log(ErrorLevel.DebugInfo, $"Response failed. License is not valid.");
                            result = null;
                            retry = false;
                        }
                    }
                    else
                    {
                        Logger.Instance.Log(ErrorLevel.ResponseError, $"Response {referenceName} returned null.");
                    }
                }
                catch (Exception ex)
                {
                    exception = ex;

                    Logger.Instance.Log(ErrorLevel.ConnectionError, $"An error occurred while calling {referenceName} from Web Service: '{ex.Message}'.");

                    CurrentConnector.Error(endpoint);

                    NotificationManager.Notify(OnAfterErpException, new OnAfterErpExceptionArgs(request, erpXmlResponse, referenceName, ex));
                }
                finally
                {
                    NotificationManager.Notify(OnAfterErpCommunication, new OnAfterErpCommunicationArgs(request, erpXmlResponse, referenceName, exception));
                }

                // no retry or xml reply from ERP then leave
                if (!retry || result != null)
                {
                    break;
                }

                // in retry mode, wait some time
                ++retryCount;
                System.Threading.Thread.Sleep(RetryInterval);
            }
            while (retryCount < MaxRetryCount);

            if ((throwException || EnableThrowExceptions) && exception != null)
            {
                throw exception;
            }
            return result;
        }

        internal static string RetrievePDF(string requestString)
        {
            Diagnostics.ExecutionTable.Current.Add("LiveIntegration.Connector.RetrievePDF START");
            string base64EncodedPDF = string.Empty;
            try
            {
                Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Request RetrievePDF sent: '{0}'.", requestString));
                base64EncodedPDF = CurrentConnector.Execute(requestString);
                Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Response RetrievePDF received: '{0}'.", base64EncodedPDF));
            }
            catch (Exception ex)
            {
                Logger.Instance.Log(ErrorLevel.ResponseError, string.Format("Response RetrievePDF returned error: '{0}'.", ex.Message));
                throw ex;
            }
            Diagnostics.ExecutionTable.Current.Add("LiveIntegration.Connector.RetrievePDF END");
            return base64EncodedPDF;
        }
    }
}