﻿using Dynamicweb.DataIntegration.Integration.ERPIntegration;
using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.LiveIntegration.EndpointMonitoring;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Dynamicweb.Ecommerce.LiveIntegration.Connectors
{
    /// <summary>
    /// Web service connector
    /// </summary>
    internal class WebServiceConnector : ConnectorBase
    {
        /// <summary>
        /// Gets the security key.
        /// </summary>
        /// <value>The security key.</value>
        private string SecurityKey => Settings.Instance.SecurityKey;

        /// <summary>
        /// Gets the URL.
        /// </summary>
        /// <value>The URL.</value>
        /// <exception cref="ArgumentException">
        /// Setup does not contain a url for the WebService. Please rerun setup.
        /// or
        /// Cannot find appropriate web service url. Check Web Service Url settings.
        /// </exception>
        internal string WebServiceUrl
        {
            get
            {
                string url = UrlHandler.Instance.GetWebServiceUrl();
                if (!string.IsNullOrEmpty(url))
                {
                    return url;
                }

                if (string.IsNullOrEmpty(Settings.Instance.WebServiceURI))
                {
                    throw new ArgumentException("Setup does not contain a url for the WebService. Please rerun setup.");
                }
                else
                {
                    throw new ArgumentException("Cannot find appropriate web service url. Check Web Service Url settings.");
                }
            }
        }

        internal override EndpointInfo GetEndpoint()
        {
            return new EndpointInfo(WebServiceUrl);
        }

        internal EndpointInfo GetEndpointInfo(string url)
        {
            return new EndpointInfo(url);
        }

        /// <summary>
        /// Determines whether the web service connection is available.
        /// </summary>
        internal override bool IsWebServiceConnectionAvailable(EndpointInfo endpoint)
        {
            if (!Settings.Instance.IsLiveIntegrationEnabled)
            {
                return false;
            }
            if (string.IsNullOrEmpty(endpoint?.WebServiceUrl))
            {
                endpoint = new EndpointInfo(WebServiceUrl);
            }
            Exception error = null;
            return ExecuteConnectionAvailableRequest(endpoint, out error);
        }

        internal override string Execute(string request)
        {
            return Execute(GetEndpointInfo(UrlHandler.Instance.GetWebServiceUrl()), request);
        }

        internal override string Execute(EndpointInfo endpoint, string request, TimeSpan responseTimeout)
        {
            string response = ErpServiceCaller.GetDataFromRequestString(endpoint.GetUrl(), SecurityKey, request, responseTimeout);
            EndpointMonitoringService.Success(endpoint);
            return response;
        }

        internal override IEnumerable<string> GetUrls(string multipleUrlsText)
        {
            return UrlHandler.Instance.GetWebServiceUrls(multipleUrlsText).Select(u => UrlHandler.Instance.GetUrl(u)).Distinct();
        }

        internal override bool IsConnectionAvailableFromBackend(string url, out string error)
        {
            error = null;
            Exception ex = null;
            if (ExecuteConnectionAvailableRequest(GetEndpointInfo(url), out ex))
            {
                return true;
            }
            else
            {
                error = $"Can not connect to the web service: {url} error: {ex?.Message}.";
                return false;
            }
        }
    }
}
