﻿using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.LiveIntegration.Connectors;
using Dynamicweb.Ecommerce.Products;
using Dynamicweb.Security.UserManagement;
using System.Collections.Generic;

namespace Dynamicweb.Ecommerce.LiveIntegration
{
    /// <summary>
    /// Helper class to provide live integration information to templates.
    /// </summary>
    public static class TemplatesHelper
    {
        /// <summary>
        /// Indicates if the system is able to connect to the ERP.
        /// </summary>
        public static bool IsWebServiceConnectionAvailable()
        {
            return Connector.IsWebServiceConnectionAvailable();
        }

        /// <summary>
        /// Determines whether the last order sync for the current context failed.
        /// </summary>
        public static bool HasErrorOnExportOrder()
        {
            bool hasErrorOnExportOrder;

            if (Context.Current != null && Context.Current.Session != null
                && bool.TryParse(Context.Current.Session["LiveIntegration.OrderExportFailed"].ToString(), out hasErrorOnExportOrder))
            {
                return hasErrorOnExportOrder;
            }

            return false;
        }

        /// <summary>
        /// Gets the ID of the last failed order.
        /// </summary>
        /// <returns>System.String.</returns>
        public static string FailedOrderId()
        {
            var o = Context.Current.Session["LiveIntegration.FailedOrderId"];
            return o?.ToString() ?? string.Empty;
        }

        /// <summary>
        /// Forces an update of the order in the ERP.
        /// </summary>
        /// <param name="orderId">The ID of the order to send to the ERP</param>
        /// <returns>Returns null if no communication has made, otherwise it returns true / false to indicate success or failure in sending the order.</returns>
        public static bool? UpdateOrder(string orderId)
        {
            return UpdateOrder(Orders.Order.GetOrderById(orderId));
        }

        /// <summary>
        /// Forces an update of the order in the ERP.
        /// </summary>
        /// <param name="order">The order to send to the ERP</param>
        /// <returns>Returns null if no communication has made, otherwise it returns true / false to indicate success or failure in sending the order.</returns>
        public static bool? UpdateOrder(Orders.Order order)
        {
            return OrderHandler.UpdateOrder(order, SubmitType.FromTemplates);
        }

        /// <summary>
        /// Pick an existing order and updates product information on it
        /// </summary>
        /// <param name="orderId">The ID of the order.</param>
        public static void UpdateStockOnOrder(string orderId)
        {
            UpdateStockOnOrder(Orders.Order.GetOrderById(orderId));
        }

        /// <summary>
        /// Updates product information on the order.
        /// </summary>
        /// <param name="order">The order.</param>
        public static void UpdateStockOnOrder(Orders.Order order)
        {
            // call LI method to check stocks in ERP
            NotificationSubscribers.NotificationSubscriberBase.UpdateProductInformation(order);
        }

        /// <summary>
        /// Updates the specified product with live information from the ERP.
        /// </summary>
        /// <param name="product">The product.</param>
        /// <param name="quantity">The quantity.</param>
        /// <param name="updateCache">Update response cache.</param>
        /// <returns><c>true</c> if product information was updated, <c>false</c> otherwise.</returns>
        public static bool UpdateProduct(Product product, double quantity, bool updateCache = false)
        {
            return Products.ProductManager.FetchProductInfos(
                new Dictionary<Product, double> { { product, quantity } },
                User.GetCurrentExtranetUser(),
                updateCache);
        }

        /// <summary>
        /// Determine if the product live information is active and should be loaded in AJAX call.
        /// </summary>
        public static bool IsLazyLoadingForProductInfoEnabled
        {
            get
            {
                return Global.IsIntegrationActive && Settings.Instance.EnableLivePrices && Connector.IsWebServiceConnectionAvailable()
                    && (Settings.Instance.LiveProductInfoForAnonymousUsers || User.GetCurrentExtranetUser() != null)
                    && Global.IsIntegrationEnabledForShop && (User.GetCurrentExtranetUser() == null || !User.GetCurrentExtranetUser().IsLivePricesDisabled)
                    && Settings.Instance.LazyLoadProductInfo;
            }
        }

    }
}